import {useForm} from "react-hook-form"
import React, {useState, useEffect} from "react";
import {format} from 'date-fns'
import "react-datepicker/dist/react-datepicker.css";
import {useClients, useCurrentUserData, useProjects, useTasks, useTimeEntries} from "../hooks";
import Row from "./Row";
import Column from "./Column";
import InputText from "./Form/InputText";
import DateSelect from "./Form/DateSelect";
import Select from "./Form/Select";
import Card from "./Card";
import TrackTime from "./TrackTime";
import Button from '@mui/material/Button';
import TagSelect from "./TagSelect";
import {addMultipleTags} from "./TagSelect";
import Switch from "./Form/Switch";
import DateTimePicker from "./Form/DateTimePicker";

const FormTimeEntry = ({task_id, client_id, project_id}) => {
	const {
		register,
		handleSubmit,
		watch,
		reset,
		formState: {errors},
		control,
		setValue,
	} = useForm()
	const [startDate, setStartDate] = useState(new Date());
	const [tags, setTags] = useState([]);
	const {createAsync} = useTimeEntries({query_params: {task_id}, disable_get: true});
	const {query: queryClients, options: optionsClients, items: clients} = useClients()
	const {query: queryProjects, options: optionsProjects, items: projects} = useProjects()
	const {query: queryTasks, options: optionsTasks, items: tasks} = useTasks({query_params: {fields: ['id', 'title', 'project_id','client_id']}})
	const {isInternal, isAdmin, query} = useCurrentUserData();
	const splitTimeString = (timeString) => {
		// Regular expressions to match different time formats
		const hoursMinutesRegex = /(\d+)\s*h\s*(\d+)\s*m/;
		const hoursRegex = /(\d+)\s*h/;
		const minutesRegex = /(\d+)\s*m/;
		const hoursColonMinutesRegex = /(\d+):(\d+)/;
		const minutesOnlyRegex = /(\d+)/;
		const hoursShortFormatRegex = /(\d+)\s*h(\d+)/;

		// Try matching each format and extracting hours and minutes
		let hours = 0;
		let minutes = 0;

		if (timeString.match(hoursMinutesRegex)) {
			const matches = timeString.match(hoursMinutesRegex);
			hours = parseInt(matches[1], 10);
			minutes = parseInt(matches[2], 10);
		} else if (timeString.match(hoursRegex)) {
			const matches = timeString.match(hoursRegex);
			hours = parseInt(matches[1], 10);
		} else if (timeString.match(minutesRegex)) {
			const matches = timeString.match(minutesRegex);
			minutes = parseInt(matches[1], 10);
		} else if (timeString.match(hoursColonMinutesRegex)) {
			const matches = timeString.match(hoursColonMinutesRegex);
			hours = parseInt(matches[1], 10);
			minutes = parseInt(matches[2], 10);
		} else if (timeString.match(hoursShortFormatRegex)) {
			const matches = timeString.match(hoursShortFormatRegex);
			hours = parseInt(matches[1], 10);
			minutes = parseInt(matches[2], 10);
		} else if (timeString.match(minutesOnlyRegex)) {
			const matches = timeString.match(minutesOnlyRegex);
			minutes = parseInt(matches[1], 10);
		}

		return {hours, minutes};
	}

	const onSubmit = data => {
		const {hours, minutes} = splitTimeString(data.time);
		data.date_start = format(new Date(data.date), 'yyyy-MM-dd HH:mm:ss');
		data.minutes = minutes;
		data.hours = hours;
		data.task_id = data.task_id ? parseInt(data.task_id.value) : null;
		data.project_id = data.project_id ? parseInt(data.project_id.value) : null;
		data.client_id = data.client_id ? parseInt(data.client_id.value) : null;
		data.internal = data.internal ? 1 : 0;
		data.private = data.private ? 1 : 0;

		if (data.hours) {
			data.minutes = parseInt(data.hours) * 60 + data.minutes;
		}

		createAsync(data).then(result => {
			addMultipleTags(result.item.id, 'time_entry', tags);
		});
	}

	const selected_client = watch("client_id");
	const selected_project = watch("project_id");
	const selected_task = watch("task_id");

	useEffect(() => {
		if (selected_task) {
			const task = tasks.find(item => item.id === selected_task.value);
			if (task) {
				const clientOption = optionsClients.find(item => item.value === task.client_id);
				if (clientOption) {
					setValue("client_id", clientOption);
				} else {
					setValue("client_id", null);
				}
				if (task.project_id) {
					const projectOption = optionsProjects.find(item => item.value === task.project_id);
					if (clientOption) {
						setValue("project_id", projectOption);
					} else {
						setValue("project_id", null);
					}
				} else {
					setValue("project_id", null);
				}
			}
		}
		return () => {

		};
	}, [selected_task, tasks, clients]);


	if (queryClients.isLoading || queryProjects.isLoading || queryTasks.isLoading) {
		return <div>Loading...</div>
	}

	let filteredOptionsProjects = optionsProjects;
	let filteredProjectIds = [];
	let filteredOptionsTasks = optionsTasks;
	let filteredTaskIds = [];

	if (selected_client) {
		filteredProjectIds = projects.filter(item => item.client_id === selected_client.value).map(item => item.id);
		filteredOptionsProjects = filteredOptionsProjects.filter(item => filteredProjectIds.includes(item.value));

		filteredTaskIds = tasks.filter(item => filteredProjectIds.includes(item.project_id)).map(item => item.id);
		filteredOptionsTasks = filteredOptionsTasks.filter(item => filteredTaskIds.includes(item.value));
	}

	if (selected_project) {
		filteredTaskIds = tasks.filter(item => item.project_id === selected_project.value).map(item => item.id);
		filteredOptionsTasks = filteredOptionsTasks.filter(item => filteredTaskIds.includes(item.value));
	}

	return (
		/* "handleSubmit" will validate your inputs before invoking "onSubmit" */
		<form onSubmit={handleSubmit(onSubmit)}>
			<TrackTime
				client_id={client_id}
				project_id={project_id}
				task_id={task_id}
				options_client={optionsClients}
				options_project={optionsProjects}
				options_tasks={optionsTasks}
				onStopTracking={(duration, data) => {
					duration.minutes = parseInt(duration.minutes);
					setValue("time", `${duration.hours}:${duration.minutes}`);
					setValue('client_id', optionsClients.find(item => item.value === data.client_id));
					setValue('project_id', optionsProjects.find(item => item.value === data.project_id));
					setValue('task_id', optionsTasks.find(item => item.value === data.task_id));
				}}
			/>
			<Card title="Create time entry">
				<Row>
					<Column>
						<InputText
							name={"time"}
							register={register}
							placeholder={"Time"}
							label={"Time"}
							defaultValue={0}
							control={control}
						/>
					</Column>
					<Column>
						<InputText
							name={"description"}
							register={register}
							placeholder={"Description"}
							label={"Description"}
							control={control}
						/>
					</Column>
					<Column>
						<DateTimePicker
							placeholder={"Date"}
							required={true}
							name={"date"}
							control={control}
							defaultValue={new Date()}
						/>
					</Column>
				</Row>
				<Row>
					<Column>
						<Select
							label="Client"
							options={optionsClients}
							placeholder={"Client"}
							name={"client_id"}
							control={control}
							defaultValue={client_id ? optionsClients.find(item => item.value === client_id) : null}
						/>
					</Column>
					<Column>
						<Select
							label="Project"
							options={filteredOptionsProjects}
							placeholder={"Project"}
							name={"project_id"}
							control={control}
							defaultValue={project_id ? optionsProjects.find(item => item.value === project_id) : null}
						/>
					</Column>
					<Column>
						<Select
							label="Task"
							options={filteredOptionsTasks}
							placeholder={"Task"}
							name={"task_id"}
							control={control}
							defaultValue={task_id ? optionsTasks.find(item => item.value === task_id) : null}
						/>
					</Column>
					<Column>
						<TagSelect
							onChangeCustom={(value) => setTags(value)}
						/>
					</Column>
					{
						(isInternal || isAdmin) && <Column>
							<Switch
								label="Internal"
								register={register}
								name="internal"
								control={control}
							/>
						</Column>
					}

					<Column>
						<Switch
							label="Private"
							register={register}
							name="private"
							control={control}
						/>
					</Column>
					<Column>
						<Button variant="outlined" type="submit">Submit</Button>
					</Column>
				</Row>
			</Card>
		</form>
	)
}
export default FormTimeEntry;
