import {useMemo, useState, useEffect} from "react"
import ColumnContainer from "./ColumnContainer"
import {
	DndContext,
	DragOverlay,
	PointerSensor,
	useSensor,
	useSensors
} from "@dnd-kit/core"
import {SortableContext, arrayMove} from "@dnd-kit/sortable"
import {createPortal} from "react-dom"
import TaskCard from "./TaskCard"
import {useTasks} from "../hooks";
import apiFetch from "@wordpress/api-fetch";
import {useQueryClient} from "@tanstack/react-query";

const defaultCols = [
	{
		id: "todo",
		title: "Todo"
	},
	{
		id: "in_progress",
		title: "In progress"
	},
	{
		id: "review",
		title: "Review"
	},
	{
		id: "done",
		title: "Done"
	}
]

function KanbanBoard({project_id, client_id}) {
	const [columns, setColumns] = useState(defaultCols)
	const columnsId = useMemo(() => columns.map(col => col.id), [columns])

	const {items, query} = useTasks({query_params: {project_id}});

	const [tasks, setTasks] = useState([])

	const [activeColumn, setActiveColumn] = useState(null)

	const [activeTask, setActiveTask] = useState(null)

	const [updateData, setUpdateData] = useState(false);

	const queryClient = useQueryClient();
	const sensors = useSensors(
		useSensor(PointerSensor, {
			activationConstraint: {
				distance: 10
			}
		})
	)


	useEffect(() => {
		if (items) {
			const tasks = items.map(item => {
				item.columnId = item.status;
				item.content = item.title;
				return item;
			})
			tasks.sort((a, b) => (a.order_index > b.order_index) ? 1 : -1)
			setTasks(tasks);
		}
		return () => {

		};
	}, [items]);

	useEffect(() => {
		if (!updateData) return;
		const data = tasks.map(task => {
			task.project_id = project_id;
			task.client_id = client_id;
			return task;
		})
		apiFetch({
			path: `/wpify-erp/v1/task/kanban`,
			method: 'POST',
			data: {
				project_id: project_id,
				items: data
			}
		}).then((result) => {
			queryClient.setQueryData(['tasks',{project_id}], result);
			setUpdateData(false);
		})
	}, [tasks, updateData]);


	if (query.isLoading) {
		return 'Loading';
	}

	return (
		<div
			className="
        m-auto
        flex

        w-full
        items-center
        overflow-x-auto
        overflow-y-hidden
        px-[40px]
    "
		>
			<DndContext
				sensors={sensors}
				onDragStart={onDragStart}
				onDragEnd={onDragEnd}
				onDragOver={onDragOver}
			>
				<div className="flex gap-4">
					<div className="flex gap-4">
						<SortableContext items={columnsId}>
							{columns.map(col => (
								<ColumnContainer
									key={col.id}
									column={col}
									createTask={createTask}
									deleteTask={deleteTask}
									updateTask={updateTask}
									tasks={tasks.filter(task => task.columnId === col.id)}
								/>
							))}
						</SortableContext>
					</div>
				</div>

				{createPortal(
					<DragOverlay>
						{activeColumn && (
							<ColumnContainer
								column={activeColumn}
								deleteColumn={deleteColumn}
								updateColumn={updateColumn}
								createTask={createTask}
								deleteTask={deleteTask}
								updateTask={updateTask}
								tasks={tasks.filter(task => task.columnId === activeColumn.id)}
							/>
						)}
						{activeTask && (
							<TaskCard
								task={activeTask}
								deleteTask={deleteTask}
								updateTask={updateTask}
							/>
						)}
					</DragOverlay>,
					document.body
				)}
			</DndContext>
		</div>
	)

	function createTask(columnId) {
		setUpdateData(true);
		const newTask = {
			id: generateId(),
			columnId,
			content: `Task ${tasks.length + 1}`,
			client_id,
			project_id,
		}

		setTasks([...tasks, newTask])
	}

	function deleteTask(id) {
		const newTasks = tasks.filter(task => task.id !== id)
		setTasks(newTasks)
	}

	function updateTask(id, content) {
		const newTasks = tasks.map(task => {
			if (task.id !== id) return task
			return {...task, content}
		})
		setTasks(newTasks)
		setUpdateData(true);
	}



	function onDragStart(event) {
		if (event.active.data.current?.type === "Column") {
			setActiveColumn(event.active.data.current.column)
			return
		}

		if (event.active.data.current?.type === "Task") {
			setActiveTask(event.active.data.current.task)
			return
		}
	}

	function onDragEnd(event) {
		setActiveColumn(null)
		setActiveTask(null)

		const {active, over} = event
		if (!over) return

		const activeId = active.id
		const overId = over.id

		if (activeId === overId) return

		const isActiveAColumn = active.data.current?.type === "Column"
		if (!isActiveAColumn) return

		setColumns(columns => {
			const activeColumnIndex = columns.findIndex(col => col.id === activeId)

			const overColumnIndex = columns.findIndex(col => col.id === overId)

			return arrayMove(columns, activeColumnIndex, overColumnIndex)
		})
	}

	function onDragOver(event) {
		const {active, over} = event
		if (!over) return

		const activeId = active.id
		const overId = over.id

		if (activeId === overId) return

		const isActiveATask = active.data.current?.type === "Task"
		const isOverATask = over.data.current?.type === "Task"

		if (!isActiveATask) return

		setUpdateData(true);
		// Im dropping a Task over another Task
		if (isActiveATask && isOverATask) {
			setTasks(tasks => {
				const activeIndex = tasks.findIndex(t => t.id === activeId)
				const overIndex = tasks.findIndex(t => t.id === overId)
				let result = null;
				if (tasks[activeIndex].columnId != tasks[overIndex].columnId) {
					// Fix introduced after video recording
					tasks[activeIndex].columnId = tasks[overIndex].columnId
					result = arrayMove(tasks, activeIndex, overIndex - 1)
				} else {
					result = arrayMove(tasks, activeIndex, overIndex);
				}

				return result;
			})
		}

		const isOverAColumn = over.data.current?.type === "Column"

		// Im dropping a Task over a column
		if (isActiveATask && isOverAColumn) {
			setTasks(tasks => {
				const activeIndex = tasks.findIndex(t => t.id === activeId)

				tasks[activeIndex].columnId = overId
				console.log("DROPPING TASK OVER COLUMN", {activeIndex})
				return arrayMove(tasks, activeIndex, activeIndex)
			})
		}
	}
}

function generateId() {
	/* Generate a random number between 0 and 10000 */
	return Math.floor(Math.random() * 10001)
}

export default KanbanBoard
