import {useNavigate, useOutletContext} from "react-router-dom";
import {MaterialReactTable} from 'material-react-table';
import {useTasks, useUsers} from "../hooks";
import React, {useEffect, useMemo, useRef, useState} from "react";
import FormNewTask from "./FormNewTask";
import {Chip, MenuItem} from '@mui/material';
import Table from "./Table";
import Button from '@mui/material/Button';
import Stack from '@mui/material/Stack';
import KanbanBoard from "./KanbanBoard";
import Card from "./Card";
import TagLabel from "./TagLabel";
import TagSelect from "./TagSelect";
import PermissionsCheck from "./PermissionsCheck";
import {format} from 'date-fns'
import AddCircleOutlineIcon from "@mui/icons-material/AddCircleOutline";
import RadioButtonUncheckedIcon from '@mui/icons-material/RadioButtonUnchecked';
import TaskAltIcon from '@mui/icons-material/TaskAlt';

const Tasks = ({project_id}) => {
	const tableInstanceRef = useRef(null) //ts
	const [queryParams, setQueryParams] = useState({project_id: project_id, current_user: true, status_not: 'done'});
	const navigate = useNavigate();
	const {items: queryTasks, query: tasksQuery, deleteItems} = useTasks({query_params: queryParams});
	const {items: users, query: usersQuery} = useUsers();
	const [addNewFormShown, setAddNewFormShown] = useState(false);
	const [view, setView] = useState('table');

	if (!project_id) {
		const {setHeader, setBreadcrumbsRoute} = useOutletContext();
		useEffect(() => {
			setHeader(() => () => <>Tasks</>);
			setBreadcrumbsRoute([{'label': 'Tasks'}]);
		}, []);
	}

	const toggleQueryParam = (paramKey, paramValue) => {
		const updatedQueryParams = {...queryParams};

		if (updatedQueryParams[paramKey] === paramValue) {
			delete updatedQueryParams[paramKey];
		} else {
			updatedQueryParams[paramKey] = paramValue;
		}

		setQueryParams(updatedQueryParams);
	};

	const QueryFilterButtons = () => {
		return <>
			<Button size="small"
					color={queryParams.current_user ? 'success' : 'inherit'}
					sx={{ml: 2}}
					onClick={() => toggleQueryParam('current_user', true)}>
				{queryParams.current_user ?
					<TaskAltIcon fontSize="inherit" sx={{mr: '3px'}}/> :
					<RadioButtonUncheckedIcon fontSize="inherit" sx={{mr: '3px'}}/>
				} My tasks
			</Button>
			<Button size="small"
					color={queryParams.status_not === 'done' ? 'success' : 'inherit'}
					onClick={() => toggleQueryParam('status_not', 'done')}>
				{queryParams.status_not === 'done' ?
					<TaskAltIcon fontSize="inherit" sx={{mr: '3px'}}/> :
					<RadioButtonUncheckedIcon fontSize="inherit" sx={{mr: '3px'}}/>
				} Only open
			</Button>
		</>
	};

	const columns = useMemo(
		() => [
			{
				header: 'Title',
				accessorFn: (row) => {
					return <span className="cursor-pointer"
								 onClick={() => navigate(`/task/${row.id}`)}>{row.title}</span>;
				},
			},
			{
				header: 'Status',
				accessorFn: (row) => {
					let chipColor = 'warning';
					if (row.status === 'done') {
						chipColor = 'success';
					}
					if (row.status === 'review') {
						chipColor = 'secondary';
					}
					if (row.status === 'in_progress') {
						chipColor = 'primary';
					}

					return <Chip color={chipColor} label={row.status} size="small"/>
				},
				accessorKey: 'status', //simple accessorKey pointing to flat data
			},
			{
				header: 'Due date',
				accessorFn: (row) => {
					if (!row.due_date || '0000-00-00 00:00:00' === row.due_date) {
						return '';
					}
					if (row.due_date && format(new Date(), 'yyyy-MM-dd HH:mm:ss') > row.due_date) {
						return <Chip color="error" label={format(new Date(row.due_date), 'dd.MM.yyyy')}/>;
					} else {
						return <Chip color="primary" label={format(new Date(row.due_date), 'dd.MM.yyyy')}/>;
					}

				}
			},
			{
				header: 'Assigned',
				accessorFn: (row) => {
					const user = users ? users.find(user => user.id === row.assigned_user_id) : null;

					return user ? user.display_name : '';
				}
			},
			{
				header: 'Tags',
				accessorFn: (row, index, returnString) => {
					if (!row.tags.length) {
						return '';
					}
					if (returnString) {
						return row.tags.map(item => item.title).join(', ');
					}
					return row.tags.map(item => <TagLabel tag={item}/>);
				},
				filterFn: 'tagsFilterFn',
				Edit: ({cell, column, table}) => {
					return <>
						<TagSelect
							object_id={cell.row.original.id}
							object_type={'task'}
							initialValue={cell.row.original.tags}
						/>
					</>
				},
			},
			{
				header: 'Project',
				accessorFn: (row) => {
					if (!row.project) {
						return '';
					}
					return <span className="cursor-pointer"
								 onClick={() => navigate(`/project/${row.project.id}`)}>{row.project.name}</span>;
				},
			},
			{
				header: 'Client',
				accessorFn: (row) => {
					if (!row.client) {
						return '';
					}
					return <span className="cursor-pointer"
								 onClick={() => navigate(`/client/${row.client.id}`)}>{row.client.title}</span>;
				},
			},
			{
				header: 'Date',
				accessorKey: 'date_created', //simple accessorKey pointing to flat data
			},

		],
		[queryTasks, users],
	);

	if (tasksQuery.isLoading || usersQuery.isLoading) {
		return 'Loading';
	}
	let tasks = queryTasks;
	if (project_id) {
		tasks = tasks.filter(item => item.project_id === project_id)
	}

	return (
		<>
			{addNewFormShown && <Card>
				<h3>Create task</h3>
				<FormNewTask
					project_id={project_id}
				/></Card>}

			<Stack direction="row" spacing={2} sx={{mb: 1}}>
				{/*<Button onClick={() => setQueryParams({...queryParams, status_not: 'done', current_user: true})}>My*/}
				{/*	open tasks</Button>*/}
				{/*<Button onClick={() => setQueryParams({})}>All tasks</Button>*/}
				<Button onClick={() => setView('table')} sx={view === 'table' ? {
					backgroundColor: 'white',
					border: 1,
					borderColor: 'grey.300'
				} : {}}>Table</Button>
				<Button onClick={() => setView('kanban')} sx={view === 'kanban' ? {
					backgroundColor: 'white',
					border: 1,
					borderBottom: 0,
					borderColor: 'grey.300'
				} : {}}>Kanban</Button>
			</Stack>

			{/*{*/}
			{/*	Object.keys(queryParams).length > 0 && <div>Global data filters: {JSON.stringify(queryParams)}</div>*/}
			{/*}*/}

			{
				view === 'table' && <><Table
					data={tasks}
					ToolbarActions={<>
						<PermissionsCheck object_id={project_id} role={'editor'} object_type={'project'} module='task'>
							<Button variant="contained" size="small"
									startIcon={<AddCircleOutlineIcon/>}
									onClick={() => setAddNewFormShown(!addNewFormShown)}>Add new task</Button>
						</PermissionsCheck>
						<QueryFilterButtons/>
					</>}
					deleteItems={deleteItems}
					columns={columns}
					tableInstanceRef={tableInstanceRef}
					filterFns={{
						tagsFilterFn: (row, id, filterValue) => {
							const strings = row.original.tags.map(tag => tag.title).join(',').toLowerCase();
							console.log(strings);
							return strings.includes(filterValue.toLowerCase());
						},
					}}
				/>
				</>

			}
			{
				view === 'kanban' && <KanbanBoard
					project_id={project_id}
				/>

			}


		</>
	);
}
export default Tasks;
