import {useNavigate, useOutletContext} from "react-router-dom";

import {useClients, useProjects, useTasks, useTimeEntries, useTimeEntry} from "../hooks";
import React, {useMemo, useRef, useState, useEffect} from "react";
import Filters from "./Filters";
import FormTimeEntry from "./FormTimeEntry";
import Table from "./Table";
import Button from '@mui/material/Button';
import InputTextControlled from "./Form/InputTextControlled";
import SelectControlled from "./Form/SelectControlled";
import TagLabel from "./TagLabel";
import TagSelect from "./TagSelect";
import SwitchControlled from "./Form/SwitchControlled";
import PermissionsCheck from "./PermissionsCheck";
import {format} from "date-fns";

export const toHoursAndMinutes = (totalMinutes) => {
	const hours = Math.floor(totalMinutes / 60);
	const minutes = totalMinutes % 60;

	return {hours, minutes};
}

const Projects = ({client_id, project_id, show_form = true}) => {
	const [queryParams, setQueryParams] = useState({
		date_from: format(new Date(), 'yyyy-MM-01 00:00:00'),
	});
	const [displayedData, setDisplayedData] = useState([]);
	const tableInstanceRef = useRef(null)
	const navigate = useNavigate();
	const {items, query: timeEntriesQuery, deleteItems} = useTimeEntries({query_params: queryParams});
	const [selectedTimeEntry, setSelectedTimeEntry] = useState(null);
	const {mutate} = useTimeEntry({id: selectedTimeEntry});
	const {options: clientsOptions} = useClients();
	const {options: projectsOptions} = useProjects();
	const {options: tasksOptions} = useTasks({query_params: {fields: ['id', 'title']}});

	const changedRowData = useRef({});

	const handleRowEdit = ({exitEditingMode, row, values}) => {
		setSelectedTimeEntry(row.original.id);
		const data = changedRowData.current;
		data.description = values.description;
		data.date_start = values.date_start;
		const time = toHoursAndMinutes(row.original.minutes);
		if (data.hasOwnProperty('edited_minutes')) {
			time.minutes = data.edited_minutes;
		}
		if (data.hasOwnProperty('edited_hours')) {
			time.hours = data.edited_hours;
		}
		if (data.hasOwnProperty('internal')) {
			data.internal = data.internal ? 1 : 0;
		}
		if (data.hasOwnProperty('private')) {
			data.private = data.private ? 1 : 0;
		}
		data.minutes = time.hours * 60 + time.minutes;

		mutate(data);
		exitEditingMode();

		changedRowData.current = {};
	};

	const columns = useMemo(
		() => [
			{
				header: 'Minutes',
				accessorFn: (row, index, returnString) => {
					const time = toHoursAndMinutes(row.minutes);
					if (returnString) {
						return `${time.hours}h ${parseInt(time.minutes) !== 0 ? time.minutes + 'm' : ''}`;
					}
					return `${time.hours}h ${parseInt(time.minutes) !== 0 ? time.minutes + 'm' : ''}`;
				},
				Edit: ({cell, column, table}) => {
					const time = toHoursAndMinutes(cell.row.original.minutes);
					return <>
						<InputTextControlled
							placeholder={'Hours'}
							type={'number'}
							defaultValue={time.hours}
							onChange={(e) => {
								changedRowData.current.edited_hours = parseInt(e.target.value)
							}}
						/>
						<InputTextControlled
							placeholder={'Minutes'}
							type={'number'}
							defaultValue={time.minutes}
							onChange={(e) => {
								changedRowData.current.edited_minutes = parseInt(e.target.value)
							}}
						/>

					</>
				},
				Footer: () => {
					const rows = tableInstanceRef.current.getRowModel().rows
					let minutes = 0;
					rows.map(item => {
						const time = toHoursAndMinutes(item.original.minutes);
						minutes += time.minutes + (time.hours * 60);
					});
					const total = toHoursAndMinutes(minutes)
					return <div>{total.hours}h {total.minutes}m</div>

				},
			},
			{
				header: 'Entry Desc',
				accessorFn: (row, index, returnString) => {
					return row.description;
				},
			},
			{
				header: 'Description',
				accessorFn: (row, index, returnString) => {
					if (!row.task) {
						return '';
					}
					return row.task?.description;
				},
			},
			{
				header: 'Task',
				accessorFn: (row, index, returnString) => {
					if (!row.task) {
						return '';
					}
					if (returnString) {
						return row.task.title;
					}
					return <span className="cursor-pointer"
								 onClick={() => navigate(`/task/${row.task.id}`)}>{row.task.title}</span>;
				},
				Edit: ({cell, column, table}) => {
					const task_id = cell.row.original.task_id;
					const selected = tasksOptions.find(item => item.value === task_id);
					return <>
						<SelectControlled
							defaultValue={selected}
							onChange={value => {
								const newValue = value ? value.value : null;
								changedRowData.current.task_id = newValue;
							}}
							placeholder={column.header}
							options={tasksOptions}
						/>
					</>
				},
				filterFn: 'rawFilterFn',
			},
			{
				header: 'Project',
				accessorFn: (row, index, returnString) => {
					if (!row.project) {
						return '';
					}
					if (returnString) {
						return row.project.name;
					}
					return <span className="cursor-pointer"
								 onClick={() => navigate(`/project/${row.project.id}`)}>{row.project.name}</span>;
				},
				Edit: ({cell, column, table}) => {
					const project_id = cell.row.original.project_id;
					const selected = projectsOptions.find(item => item.value === project_id);
					return <>
						<SelectControlled
							defaultValue={selected}
							onChange={value => {
								const newValue = value ? value.value : null;
								changedRowData.current.project_id = newValue;
							}}
							placeholder={column.header}
							options={projectsOptions}
						/>
					</>
				},
				filterFn: 'rawFilterFn',
			},
			{
				header: 'Client',
				sortingFn: (rowA, rowB, columnId) => {
					return rowA.original.client_id > rowB.original.client_id ? 1 : -1;
				},
				accessorFn: (row, index, returnString) => {
					if (!row.client) {
						return '';
					}
					if (returnString) {
						return row.client.title;
					}
					return <span className="cursor-pointer"
								 onClick={() => navigate(`/client/${row.client.id}`)}>{row.client.title}</span>;
				},
				exportCsv: (row) => {
					if (!row.client) {
						return '';
					}
					return row.client.title;
				},
				Edit: ({cell, column, table}) => {
					const client_id = cell.row.original.client_id;
					const selected = clientsOptions.find(item => item.value === client_id);
					return <>
						<SelectControlled
							defaultValue={selected}
							onChange={value => {
								const newValue = value ? value.value : null;
								changedRowData.current.client_id = newValue;
							}}
							placeholder={column.header}
							options={clientsOptions}
						/>
					</>
				},
				filterFn: 'rawFilterFn',
			},
			{
				header: 'Tags',
				accessorFn: (row, index, returnString) => {
					if (!row.tags.length) {
						return '';
					}
					if (returnString) {
						return row.tags.map(item => item.title).join(', ');
					}
					return row.tags.map(item => <TagLabel tag={item}/>);
				},
				Edit: ({cell, column, table}) => {
					return <>
						<TagSelect
							object_id={cell.row.original.id}
							object_type={'time_entry'}
							initialValue={cell.row.original.tags}
						/>
					</>
				},
			},
			{
				header: 'Date',
				accessorKey: 'date_start',
			},
			{
				header: 'User',
				accessorFn: (row) => {
					return row.user.display_name;
				},
			},
			{
				header: 'Internal',
				accessorFn: (row) => {
					return row.internal;
				},
				Edit: ({cell, column, table}) => {
					const checked = cell.row.original.internal;
					return <>
						<SwitchControlled
							defaultValue={checked}
							onChange={value => {
								const newValue = value ? 1 : 0;
								changedRowData.current.internal = newValue;
							}}
							placeholder={column.header}
						/>
					</>
				},
			},
			{
				header: 'Private',
				accessorFn: (row) => {
					return row.private;
				},
				Edit: ({cell, column, table}) => {
					const checked = cell.row.original.private;
					return <>
						<SwitchControlled
							defaultValue={checked}
							onChange={value => {
								const newValue = value ? 1 : 0;
								changedRowData.current.private = newValue;
							}}
							placeholder={column.header}
						/>
					</>
				},

			},

		],
		[items, tasksOptions, clientsOptions, projectsOptions, changedRowData],
	);


	useEffect(() => {
		if (items) {
			setDisplayedData(items);
		}
	}, [items]);

	const {setHeader, setBreadcrumbsRoute} = useOutletContext();
	useEffect(() => {
		setHeader(() => () => <>Time entries</>);
		setBreadcrumbsRoute([{'label': 'Time entries'}]);
	}, []);


	if (timeEntriesQuery.isLoading) {
		return 'Loading';
	}

	const handleFilters = (filters) => {
		const params = {...queryParams}
		if (filters.date_from) {
			params.date_from = filters.date_from;
		}
		if (filters.date_to) {
			params.date_to = filters.date_to;
		}
		setQueryParams(params);
	}

	const handleClearFilters = () => {
		setQueryParams({});
	}


	return (
		<>
			<FormTimeEntry/>
			<Filters
				onFilter={handleFilters}
				onClear={handleClearFilters}
			>
				<Button onClick={() => setQueryParams({current_user: true, ...queryParams})}>My entries</Button>


			</Filters>
			<Table
				data={displayedData}
				deleteItems={deleteItems}
				columns={columns}
				tableInstanceRef={tableInstanceRef}
				enableEditing={true}
				onEditingRowSave={handleRowEdit}
				manualPagination={true}
			/>
		</>
	);
}
export default Projects;
