import apiFetch from '@wordpress/api-fetch';
import {useMutation, useQuery, useQueryClient} from "@tanstack/react-query";
import {addQueryArgs} from "@wordpress/url";
import {toast} from 'react-toastify';
import {useState} from "react";
import {useNavigate, useParams} from "react-router-dom";
import JSONCrush from 'jsoncrush'
import {useLocation, matchPath} from 'react-router-dom';

export const useToastify = () => {
	const [toastId, setToastId] = useState(null);
	return {
		loading: () => {
			setToastId(toast.loading("Please wait..."));
		},
		success: (message) => {
			toast.update(toastId, {
				render: message ? message : 'Success!',
				type: "success",
				isLoading: false,
				autoClose: 2000
			});
		}
	}
}
// Create a client
export const getProjects = ({queryKey}) => {
	const [_key, {client_id}] = queryKey
	const path = addQueryArgs('/wpify-erp/v1/project', {client_id});

	return apiFetch({path}).then((items) => {
		return items;
	});
}


const getHookCollection = (args = {}) => {
	if (typeof args.query_key === 'string' && args.hasOwnProperty('query_params')) {
		args.query_key = [args.query_key, args.query_params];
	}

	if (args.hasOwnProperty('query_params')) {
		console.log(args.query_params);
		args.endpoint = addQueryArgs(args.endpoint, args.query_params)
	}

	if (typeof args.query_key === 'string') {
		args.query_key = [args.query_key];
	}

	const {loading, success} = useToastify();

	const query = !args?.disable_get ? useQuery({
		queryKey: args.query_key, queryFn: () => apiFetch({path: `/wpify-erp/v1/${args.endpoint}`}).then((items) => {
			return items;
		})
	}) : null;
	const queryClient = useQueryClient();
	const mutation = useMutation({
		mutationFn: (data) => apiFetch({
			path: `/wpify-erp/v1/${args.endpoint}`,
			method: 'POST',
			data
		}).then((result) => {
			return result;
		}),
		onMutate: (data) => {
			loading();
		},
		onSuccess: (data, variables) => {
			queryClient.setQueryData(args.query_key, data);
			success();

			if (args.onCreateSuccess) {
				args.onCreateSuccess(data, queryClient);
			}
		},
	})
	const mutationDelete = useMutation({
		mutationFn: (data) => apiFetch({
			path: `/wpify-erp/v1/${args.endpoint}`,
			method: 'DELETE',
			data
		}).then((result) => {
			return result;
		}),
		onMutate: (data) => {
			loading();
		},
		onSuccess: (data, variables) => {
			queryClient.setQueryData(args.query_key, data)

			success();
		},
	})

	return {
		query,
		items: query?.data?.items,
		mutation,
		create: mutation.mutate,
		createAsync: mutation.mutateAsync,
		deleteItems: mutationDelete.mutate,
		options: Array.isArray(query?.data?.items) ? query?.data?.items?.map(item => ({
			value: item.id,
			label: item?.title ? item.title : item.name
		})) : [],
	}
}

export const getHookSingle = (args = {}) => {
	args.query_key = args.query_key.map(val => val ? val : '')
	const {loading, success} = useToastify();
	const query = useQuery({
		queryKey: args.query_key, queryFn: ({queryKey}) => {
			const [_key, {id}] = queryKey
			return id ? apiFetch({path: `/wpify-erp/v1/${args.endpoint}/${args.id}`}).then((data) => {
				return data;
			}) : null;
		}
	})
	const queryClient = useQueryClient()
	const mutation = useMutation({
		mutationFn: (data) => apiFetch({
			path: `/wpify-erp/v1/${args.endpoint}/${args.id}`,
			method: 'PUT',
			data
		}).then((result) => {
			return result;
		}),
		onMutate: (data) => {
			loading();
		},
		onSuccess: (data, variables) => {
			queryClient.setQueryData(args.query_key, data)
			success();
			if (args.onUpdateSuccess) {
				args.onUpdateSuccess(data, queryClient);
			}
		},
	})

	return {
		query,
		item: query?.data?.item,
		mutation,
		mutate: mutation.mutate
	}
}
export const useProjects = (args = {}) => getHookCollection({
	endpoint: 'project',
	query_key: 'projects',
	query_params: {client_id: args.client_id},
	...args
});

export const useTasks = (args = {}) => getHookCollection({
	endpoint: 'task',
	query_key: 'tasks',
	onCreateSuccess: (data, queryClient) => {
		queryClient.invalidateQueries({queryKey: ['tasks']})
	},
	...args
});

export const useTimeEntries = (args = {}) => getHookCollection({
	endpoint: 'time-entry',
	query_key: 'time-entries',
	onCreateSuccess: (data, queryClient) => {
		queryClient.invalidateQueries({queryKey: ['task']})
		queryClient.invalidateQueries({queryKey: ['time-entries']})
	},
	...args
});

export const useClients = (args = {}) => getHookCollection({
	endpoint: 'client',
	query_key: 'clients',
	...args
});
export const useTaskStatuses = (args = {}) => getHookCollection({
	endpoint: 'enum/task-status',
	query_key: 'task-statuses',
	...args
});

export const useUsers = (args = {}) => getHookCollection({
	endpoint: 'user',
	query_key: 'users',
	...args
});

export const usePermissions = (args = {}) => getHookCollection({
	endpoint: 'permission',
	query_key: 'permissions',
	...args
});

export const useCurrentUser = (args = {}) => getHookSingle({
	endpoint: 'user',
	id: 'current',
	query_key: ['user', {id: 'current'}],
	...args
});
export const useDocuments = (args = {}) => getHookCollection({
	endpoint: 'document',
	query_key: 'documents',
	...args
});
export const useEnums = (args = {}) => getHookCollection({
	endpoint: 'enum',
	query_key: 'enums',
	...args
});

export const useTaskComments = (args = {}) => getHookCollection({
	endpoint: `task/${args.id}/comment`,
	query_key: 'task-comments',
	disable_get: true,
	onCreateSuccess: (data, queryClient) => {
		queryClient.invalidateQueries({queryKey: ['task']})
	},
	...args
});

export const useTags = (args = {}) => getHookCollection({
	endpoint: 'tag',
	query_key: 'tags',
	...args
});

export const useTask = (args = {}) => getHookSingle({
	endpoint: 'task',
	query_key: ['task', {id: args.id}],
	...args
})

export const useTimeEntry = (args = {}) => getHookSingle({
	endpoint: 'time-entry',
	query_key: ['time_entry', {id: args.id}],
	onUpdateSuccess: (data, queryClient) => {
		queryClient.invalidateQueries({queryKey: ['time-entries']})
	},
	...args
})
export const usePermission = (args = {}) => getHookSingle({
	endpoint: 'permission',
	query_key: ['permission', {id: args.id}],
	onUpdateSuccess: (data, queryClient) => {
		queryClient.invalidateQueries({queryKey: ['permissions']})
	},
	...args
})

export const useProject = (args = {}) => getHookSingle({
	endpoint: 'project',
	query_key: ['project', {id: args.id}],
	...args
})

export const useClient = (args = {}) => getHookSingle({
	endpoint: 'client',
	query_key: ['project', {id: args.id}],
	...args
})

export const useDocument = (args = {}) => getHookSingle({
	endpoint: 'document',
	query_key: ['document', {id: args.id}],
	...args
})

export const useUrlParams = () => {
	const {"*": splat, ...params} = useParams();
	const location = useLocation();
	const navigate = useNavigate();
	const cleanUrl = location.pathname.replace(/\(.*$/, '');

	const encodeParams = (params) => {
		return JSONCrush.crush(JSON.stringify(params));
	}

	let decodedSplat = {}
	if (splat) {
		try {
			decodedSplat = JSON.parse(JSONCrush.uncrush(splat));
		} catch (err) {

		}
	}

	return {
		splat,
		decodedSplat,
		encodeParams: encodeParams,
		params,
		addSplatToUrl: (newParams) => {
			const newSplat = encodeParams({...decodedSplat, ...newParams});
			navigate(`${cleanUrl.replace(/\/?(\?|#|$)/, '/$1')}${newSplat}`)
		}
	}
}

export const useCurrentUserData = () => {
	const {item, mutation, mutate, query} = useCurrentUser();
	const getRoles = (role) => {
		const roles = [role];
		if (role === 'editor') {
			roles.push('reader');
		}
		if (role === 'manager') {
			roles.push('reader');
			roles.push('editor');
		}
		return roles
	}
	return {
		item,
		mutation,
		mutate,
		query,
		isAdmin: item?.roles.includes('administrator'),
		isInternal: item?.roles.includes('wpify_erp_internal'),
		userCan: (role, object_type, object_id, module, permissionRole) => {
			if (query.isLoading) {
				return false;
			}
			if (item.roles.includes('administrator')) {
				return true;
			}
			if (permissionRole) {
				const checkRoles = getRoles(permissionRole);
				return checkRoles.includes(role);
			}

			// Administrator
			if (item.roles.includes('administrator')) {
				return true;
			}

			// No permissions at all
			if (!item.permissions.length) {
				return false;
			}

			let found = false;
			item.permissions.map(permission => {
				const roles = getRoles(permission.role);

				if (module && permission?.context?.modules?.[module]?.role) {
					if (permission.context.modules[module].role === 'none') {
						return false;
					}

					const moduleRoles = getRoles(permission.context.modules[module].role);
					if (!moduleRoles.includes(role)) {
						return false;
					} else {
						found = true;
						return true;
					}
				}

				if (roles.includes(role) && permission.object_type === object_type && parseInt(permission.object_id) === parseInt(object_id)) {
					found = true;
					return true;
				}
			});

			return found;

		}
	}
}
