<?php

namespace WpifyErp\Abstracts;

use WP_REST_Controller;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;
use WpifyErp\Controllers\ClientController;
use WpifyErp\Controllers\ProjectController;
use WpifyErp\Controllers\TaskController;
use WpifyErp\Enums\TaskStatuses;
use WpifyErp\Managers\ApiManager;
use WpifyErp\Repositories\ActivityLogRepository;
use WpifyErp\Repositories\AttachmentRepository;
use WpifyErp\Repositories\ClientRepository;
use WpifyErp\Repositories\ProjectRepository;
use WpifyErp\Repositories\TaskCommentRepository;
use WpifyErp\Repositories\TaskRepository;
use WpifyErp\Repositories\TimeEntryRepository;

abstract class AbstractRest extends WP_REST_Controller {
	abstract public function base(): string;

	public function __construct() {
		add_action( 'rest_api_init', [ $this, 'register_routes' ] );
	}

	/**
	 * Register the routes for the objects of the controller.
	 */
	public function register_routes() {
		register_rest_route(
			ApiManager::ENDPOINT,
			$this->base(),
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'create_item' ),
					'args'                => array(),
					'permission_callback' => [ $this, 'create_item_permissions_check' ],
				),
				array(
					'methods'             => WP_REST_Server::READABLE,
					'callback'            => array( $this, 'get_items' ),
					'args'                => array(),
					'permission_callback' => [ $this, 'get_items_permissions_check' ],
				),
				array(
					'methods'             => WP_REST_Server::DELETABLE,
					'callback'            => array( $this, 'delete_item' ),
					'args'                => array(),
					'permission_callback' => [ $this, 'delete_item_permissions_check' ],
				),

			)
		);
		register_rest_route(
			ApiManager::ENDPOINT,
			$this->base() . '/(?P<id>\d+)',
			array(
				array(
					'methods'             => WP_REST_Server::READABLE,
					'callback'            => array( $this, 'get_item' ),
					'permission_callback' => [ $this, 'get_item_permissions_check' ],
				),
				array(
					'methods'             => 'PUT',
					'callback'            => array( $this, 'update_item' ),
					'permission_callback' => [ $this, 'update_item_permissions_check' ],
				),
			)
		);
		register_rest_route(
			ApiManager::ENDPOINT,
			$this->base() . '/order-index',
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'save_order_index' ),
					'permission_callback' => [ $this, 'update_item_permissions_check' ],
				),
			)
		);
	}

	/**
	 * Add box to cart
	 *
	 * @param  WP_REST_Request  $request  Full data about the request.
	 *
	 * @return WP_REST_Response
	 */
	public function create_item( $request ) {
		$item       = $this->repository->create( $request->get_params() );
		$item       = $this->create_item_data( $item, $request );
		$item       = $this->repository->save( $item );
		$item_array = $item->to_array();
		$result     = array_merge( [ 'item' => $item_array ], $this->get_items( $request )->get_data() );

		do_action( 'erp_after_rest_create_item', $item, $request, $this->base() );
		erp_add_log( 'create', $item['id'], str_replace( '-', '_', $this->base() ), $request->get_params() );

		return new WP_REST_Response( $result, 201 );
	}


	public function update_item( $request ) {
		$item = $this->repository->get( $request->get_param( 'id' ) );
		foreach ( $request->get_params() as $key => $value ) {
			if ( $key === 'id' ) {
				continue;
			}
			if ( ! property_exists( $item, $key ) ) {
				continue;
			}

			$item->{$key} = $value;
		}

		$this->repository->save( $item );
		erp_add_log( 'update', $item['id'], str_replace( '-', '_', $this->base() ), $request->get_params(), $item );

		return $this->get_item( $request );
	}

	public function delete_item( $request ) {
		foreach ( $request->get_param( 'ids' ) as $id ) {
			$item = $this->repository->get( $id );
			erp_add_log( 'delete', (int) $id, str_replace( '-', '_', $this->base() ), $request->get_params() );
			$this->repository->soft_delete( $item );
		}


		return $this->get_items( $request );
	}

	public function get_items( $request ) {
		$args = [];
		if ( method_exists( $this->repository, 'build_args' ) ) {
			$args = $this->repository->build_args( $request->get_params(), $request );
		}

		$items = $this->controller->get_items( $args, $request->get_param('fields') ?? [] );

		return new WP_REST_Response( [ 'items' => $items ] );
	}

	public function save_order_index( $request ) {
		foreach ( $request->get_param( 'items' ) as $key => $item ) {
			$model              = $this->repository->get( $item['id'] );
			$model->order_index = $key;
			$this->repository->save( $model );
		}

		return $this->get_items( $request );
	}

	public function get_item( $request ) {
		$item = $this->controller->get_item( (int) $request->get_param( 'id' ) );

		return new WP_REST_Response( [ 'item' => $item ] );
	}

	public function create_item_data( $item, $request ) {
		return $item;
	}

	public function get_items_permissions_check( $request ) {
		return is_user_logged_in();
	}

	public function create_item_permissions_check( $request ) {
		return is_user_logged_in();
	}

	public function get_item_permissions_check( $request ) {
		return is_user_logged_in();
	}

	public function update_item_permissions_check( $request ) {
		return is_user_logged_in();
	}

	public function delete_item_permissions_check( $request ) {
		return is_user_logged_in();
	}
}
