<?php

namespace WpifyErp\Api;

use Ramsey\Uuid\Uuid;
use WP_REST_Controller;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;
use WpifyErp\Abstracts\AbstractRest;
use WpifyErp\Controllers\ProjectController;
use WpifyErp\Controllers\TaskController;
use WpifyErp\Enums\TaskStatuses;
use WpifyErp\Managers\ApiManager;
use WpifyErp\Models\TaskModel;
use WpifyErp\Repositories\AttachmentRepository;
use WpifyErp\Repositories\ProjectRepository;
use WpifyErp\Repositories\TaskCommentRepository;
use WpifyErp\Repositories\TaskRepository;
use WpifyErp\Repositories\TimeEntryRepository;

class TaskApi extends AbstractRest {
	public function __construct(
		public TaskController $controller,
		public TaskRepository $repository,
		private TaskCommentRepository $task_comment_repository,
		private TimeEntryRepository $time_entry_repository,
		private AttachmentRepository $attachment_repository,
		private ProjectRepository $project_repository
	) {
		parent::__construct();
		add_action( 'rest_api_init', [ $this, 'register_custom_routes' ] );
	}

	/**
	 * Register the routes for the objects of the controller.
	 */
	public function register_custom_routes() {
		register_rest_route(
			ApiManager::ENDPOINT,
			'task/kanban',
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'update_kanban' ),
					'permission_callback' => [ $this, 'update_item_permissions_check' ],
				),
			)
		);
		register_rest_route(
			ApiManager::ENDPOINT,
			'task/(?P<id>\d+)/comment',
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'create_comment' ),
					'permission_callback' => [ $this, 'update_item_permissions_check' ],
				),
			)
		);
		register_rest_route(
			ApiManager::ENDPOINT,
			'task/(?P<id>\d+)/time-entry',
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'create_time_entry' ),
					'permission_callback' => [ $this, 'update_item_permissions_check' ],
				),
			)
		);
		register_rest_route(
			ApiManager::ENDPOINT,
			'task/(?P<id>\d+)/attachment',
			array(
				array(
					'methods'             => WP_REST_Server::CREATABLE,
					'callback'            => array( $this, 'create_attachment' ),
					'permission_callback' => [ $this, 'update_item_permissions_check' ],
				),
			)
		);
	}

	/**
	 * Add box to cart
	 *
	 * @param  WP_REST_Request  $request  Full data about the request.
	 *
	 * @return WP_REST_Response
	 */
	public function update_kanban( $request ) {
		foreach ( $request->get_param( 'items' ) as $key => $item ) {
			$task = $this->repository->get( $item['id'] );
			if ( ! $task ) {
				// Create a new task
				$task             = $this->repository->create();
				$task->project_id = $item['project_id'];
				if ( ! empty( $item['project_id'] ) ) {
					$project = $this->project_repository->get( $item['project_id'] );
					if ( $project ) {
						$task->client_id = $project->client_id;
					}
				}
			}
			$task->title       = $item['content'];
			$task->status      = $item['columnId'];
			$task->order_index = $key;
			$this->repository->save( $task );
		}

		return $this->get_items( $request );
	}

	/**
	 * Add box to cart
	 *
	 * @param  WP_REST_Request  $request  Full data about the request.
	 *
	 * @return WP_REST_Response
	 */
	public function create_comment( $request ) {
		$item              = $this->task_comment_repository->create();
		$item->description = $request->get_param( 'description' );
		$item->private     = $request->get_param( 'private' );
		$item->internal    = $request->get_param( 'internal' );
		$item->task_id     = (int) $request->get_param( 'id' );
		$item->user_id     = get_current_user_id();
		$this->task_comment_repository->save( $item );
		erp_add_log( 'create', $item->id, 'comment', $request->get_params() );

		return new WP_REST_Response( $this->get_item( $request ), 201 );
	}

	/**
	 * Add box to cart
	 *
	 * @param  WP_REST_Request  $request  Full data about the request.
	 *
	 * @return WP_REST_Response
	 */
	public function create_time_entry( $request ) {
		$item              = $this->time_entry_repository->create();
		$item->description = $request->get_param( 'description' );
		$item->task_id     = (int) $request->get_param( 'task_id' );
		$item->user_id     = get_current_user_id();
		$item->minutes     = (int) $request->get_param( 'minutes' );
		$item->date_start  = date( 'Y-m-d H:i:s', strtotime( $request->get_param( 'date_start' ) ) );
		$this->time_entry_repository->save( $item );

		return new WP_REST_Response( $this->get_item( $request ), 201 );
	}


	public function create_attachment( $request ) {
		$file_name = $_FILES['file']['name'];
		$file_size = $_FILES['file']['size'];
		$file_tmp  = $_FILES['file']['tmp_name'];
		$file_type = $_FILES['file']['type'];
		$dir       = wp_upload_dir()['basedir'] . '/wpify-erp';
		if ( ! is_dir( $dir ) ) {
			mkdir( $dir );
		}

		$extension = pathinfo( $file_name, PATHINFO_EXTENSION );
		$filename  = sprintf( '%s.%s', Uuid::uuid4()->toString(), $extension );
		$subdir    = date( 'Y-m-d' );
		if ( ! is_dir( trailingslashit( $dir ) . $subdir ) ) {
			mkdir( trailingslashit( $dir ) . $subdir );
		}
		$path          = sprintf( '%s/%s', $subdir, $filename );
		$complete_path = trailingslashit( $dir ) . $path;

		if ( move_uploaded_file( $file_tmp, $complete_path ) ) {
			$item              = $this->attachment_repository->create();
			$item->user_id     = get_current_user_id();
			$item->path        = $path;
			$item->object_type = 'task';
			$item->object_id   = (int) $request->get_param( 'id' );
			$this->attachment_repository->save( $item );
		}

		return new WP_REST_Response( $this->get_item( $request ) );
	}

	public function base(): string {
		return 'task';
	}

	/**
	 * @param TaskModel $item
	 * @param $request
	 *
	 * @return mixed
	 */
	public function create_item_data( $item, $request ) {
		$item->status  = TaskStatuses::TODO;
		$item->user_id = get_current_user_id();

		return $item;
	}
}
