<?php

namespace WpifyErp\Features;

use League\HTMLToMarkdown\HtmlConverter;
use Ramsey\Uuid\Uuid;
use Webklex\PHPIMAP\ClientManager;
use WpifyErp\Enums\TaskStatuses;
use WpifyErp\Repositories\AttachmentRepository;
use WpifyErp\Repositories\TagRelationshipRepository;
use WpifyErp\Repositories\TaskCommentRepository;
use WpifyErp\Repositories\TaskRepository;

class Imap {
	public function __construct(
		private TaskRepository $task_repository,
		private AttachmentRepository $attachment_repository,
		private TagRelationshipRepository $tag_relationship_repository,
		private TaskCommentRepository $task_comment_repository
	) {
	}

	public function handle_inbox() {
		$cm = new ClientManager( $this->get_imap_config() );

		/** @var \Webklex\PHPIMAP\Client $client */
		$client = $cm->account( 'default' );

		/** @var \Webklex\PHPIMAP\Support\FolderCollection $folders */
		$folder = $client->getFolderByPath( 'INBOX' );

		//Get all Messages of the current Mailbox $folder
		/** @var \Webklex\PHPIMAP\Support\MessageCollection $messages */
		$messages = $folder->messages()->unseen()->get();
		/** @var \Webklex\PHPIMAP\Message $message */
		foreach ( $messages as $message ) {
			$from       = $message->getFrom();
			$from_email = $from[0]->mail;
			$from_name  = $from[0]->personal;

			$content   = $message->getHTMLBody() ?: $message->getTextBody();
			$converter = new HtmlConverter( array( 'strip_tags' => true ) );
			$content   = $converter->convert( $content );
			$pattern   = '/----(.*?)----/'; // This regular expression matches content between "----" delimiters
			$task_id   = null;

			if ( preg_match( $pattern, $content, $matches ) ) {
				$task_id              = (int) $matches[1];
				$comment              = $this->task_comment_repository->create();
				$comment->task_id     = $task_id;
				$comment->description = $content;
				$this->task_comment_repository->save( $comment );
				$task = $this->task_repository->get( $task_id );
				if ( $task ) {
					$task->status = TaskStatuses::TODO;
					$this->task_repository->save( $task );
				}
			} else {
				$task              = $this->task_repository->create();
				$task->internal    = 1;
				$task->title       = $message->getSubject()->get();
				$task->description = $content;
				$task->status      = TaskStatuses::TODO;
				$task->due_date    = date( 'Y-m-d H:i:s', strtotime( '+2 days' ) );
				$task->context     = [
					'email_from' => $from_email,
					'name_from'  => $from_name,
				];

				$task    = $this->task_repository->save( $task );
				$task_id = $task->id;

				$tag_relationship              = $this->tag_relationship_repository->create();
				$tag_relationship->object_id   = $task_id;
				$tag_relationship->object_type = 'task';
				$tag_relationship->tag_id      = 42;
				$this->tag_relationship_repository->save( $tag_relationship );    // Create new ticket
			}

			if ( $task_id ) {
				$message->setFlag( 'Seen' );
				/** @var \Webklex\PHPIMAP\Support\AttachmentCollection $attachments */
				foreach ( $message->getAttachments() as $attachment ) {
					/** @var \Webklex\PHPIMAP\Attachment $attachment */
					$extension = $attachment->getExtension();
					$filename  = sprintf( '%s.%s', Uuid::uuid4()->toString(), $extension );
					$subdir    = date( 'Y-m-d' );
					$dir       = wp_upload_dir()['basedir'] . '/wpify-erp';
					if ( ! is_dir( trailingslashit( $dir ) . $subdir ) ) {
						mkdir( trailingslashit( $dir ) . $subdir );
					}

					$path = sprintf( '%s/%s', $subdir, $filename );
					$attachment->save( $dir . '/' . $subdir . '/', $filename );


					$item              = $this->attachment_repository->create();
					$item->user_id     = get_current_user_id();
					$item->path        = $path;
					$item->object_type = 'task';
					$item->object_id   = $task_id;
					$this->attachment_repository->save( $item );
				}
			}
		}
	}

	public function get_imap_config() {
		return [

			/*
			|--------------------------------------------------------------------------
			| Default date format
			|--------------------------------------------------------------------------
			|
			| The default date format is used to convert any given Carbon::class object into a valid date string.
			| These are currently known working formats: "d-M-Y", "d-M-y", "d M y"
			|
			*/
			'date_format' => 'd-M-Y',

			/*
			|--------------------------------------------------------------------------
			| Default account
			|--------------------------------------------------------------------------
			|
			| The default account identifier. It will be used as default for any missing account parameters.
			| If however the default account is missing a parameter the package default will be used.
			| Set to 'false' [boolean] to disable this functionality.
			|
			*/
			'default'     => 'default',

			'{mail.yotlix.cz:993/imap/ssl}INBOX', // IMAP server and mailbox folder
			'supporttest@wpify.io', // Username for the before configured mailbox
			'supporttest', // Password for the before configured username
			__DIR__, // Directory, where attachments will be saved (optional)
			'UTF-8', // Server encoding (optional)
			true, // Trim leading/ending whitespaces of IMAP path (optional)
			/*
			|--------------------------------------------------------------------------
			| Available accounts
			|--------------------------------------------------------------------------
			|
			| Please list all IMAP accounts which you are planning to use within the
			| array below.
			|
			*/
			'accounts'    => [

				'default' => [
					'host'           => 'mail.yotlix.cz',
					'port'           => 993,
					'protocol'       => 'imap',
					'encryption'     => 'ssl', // Supported: false, 'ssl', 'tls'
					'validate_cert'  => false,
					'username'       => 'supporttest@wpify.io',
					'password'       => 'supporttest',
					'authentication' => null,
					'proxy'          => [
						'socket'          => null,
						'request_fulluri' => false,
						'username'        => null,
						'password'        => null,
					],
					"timeout"        => 30,
					"extensions"     => [],
				],

			],

			/*
			|--------------------------------------------------------------------------
			| Available IMAP options
			|--------------------------------------------------------------------------
			|
			| Available php imap config parameters are listed below
			|   -Delimiter (optional):
			|       This option is only used when calling $oClient->
			|       You can use any supported char such as ".", "/", (...)
			|   -Fetch option:
			|       IMAP::FT_UID  - Message marked as read by fetching the body message
			|       IMAP::FT_PEEK - Fetch the message without setting the "seen" flag
			|   -Fetch sequence id:
			|       IMAP::ST_UID  - Fetch message components using the message uid
			|       IMAP::ST_MSGN - Fetch message components using the message number
			|   -Body download option
			|       Default TRUE
			|   -Flag download option
			|       Default TRUE
			|   -Soft fail
			|       Default FALSE - Set to TRUE if you want to ignore certain exception while fetching bulk messages
			|   -RFC822
			|       Default TRUE - Set to FALSE to prevent the usage of \imap_rfc822_parse_headers().
			|                      See https://github.com/Webklex/php-imap/issues/115 for more information.
			|   -Debug enable to trace communication traffic
			|   -UID cache enable the UID cache
			|   -Fallback date is used if the given message date could not be parsed
			|   -Boundary regex used to detect message boundaries. If you are having problems with empty messages, missing
			|       attachments or anything like this. Be advised that it likes to break which causes new problems..
			|   -Message key identifier option
			|       You can choose between the following:
			|       'id'     - Use the MessageID as array key (default, might cause hickups with yahoo mail)
			|       'number' - Use the message number as array key (isn't always unique and can cause some interesting behavior)
			|       'list'   - Use the message list number as array key (incrementing integer (does not always start at 0 or 1)
			|       'uid'    - Use the message uid as array key (isn't always unique and can cause some interesting behavior)
			|   -Fetch order
			|       'asc'  - Order all messages ascending (probably results in oldest first)
			|       'desc' - Order all messages descending (probably results in newest first)
			|   -Disposition types potentially considered an attachment
			|       Default ['attachment', 'inline']
			|   -Common folders
			|       Default folder locations and paths assumed if none is provided
			|   -Open IMAP options:
			|       DISABLE_AUTHENTICATOR - Disable authentication properties.
			|                               Use 'GSSAPI' if you encounter the following
			|                               error: "Kerberos error: No credentials cache
			|                               file found (try running kinit) (...)"
			|                               or ['GSSAPI','PLAIN'] if you are using outlook mail
			|   -Decoder options (currently only the message subject and attachment name decoder can be set)
			|       'utf-8' - Uses imap_utf8($string) to decode a string
			|       'mimeheader' - Uses mb_decode_mimeheader($string) to decode a string
			|
			*/
			'options'     => [
				'delimiter'      => '/',
				'fetch'          => \Webklex\PHPIMAP\IMAP::FT_PEEK,
				'sequence'       => \Webklex\PHPIMAP\IMAP::ST_UID,
				'fetch_body'     => true,
				'fetch_flags'    => true,
				'soft_fail'      => false,
				'rfc822'         => true,
				'debug'          => false,
				'uid_cache'      => true,
				// 'fallback_date' => "01.01.1970 00:00:00",
				'boundary'       => '/boundary=(.*?(?=;)|(.*))/i',
				'message_key'    => 'list',
				'fetch_order'    => 'asc',
				'dispositions'   => [ 'attachment', 'inline' ],
				'common_folders' => [
					"root"  => "INBOX",
					"junk"  => "INBOX/Junk",
					"draft" => "INBOX/Drafts",
					"sent"  => "INBOX/Sent",
					"trash" => "INBOX/Trash",
				],
				'decoder'        => [
					'message'    => 'utf-8', // mimeheader
					'attachment' => 'utf-8', // mimeheader
				],
				'open'           => [
					// 'DISABLE_AUTHENTICATOR' => 'GSSAPI'
				],
			],

			/*
			|--------------------------------------------------------------------------
			| Available flags
			|--------------------------------------------------------------------------
			|
			| List all available / supported flags. Set to null to accept all given flags.
			 */
			'flags'       => [ 'recent', 'flagged', 'answered', 'deleted', 'seen', 'draft' ],

			/*
			|--------------------------------------------------------------------------
			| Available events
			|--------------------------------------------------------------------------
			|
			 */
			'events'      => [
				"message" => [
					'new'      => \Webklex\PHPIMAP\Events\MessageNewEvent::class,
					'moved'    => \Webklex\PHPIMAP\Events\MessageMovedEvent::class,
					'copied'   => \Webklex\PHPIMAP\Events\MessageCopiedEvent::class,
					'deleted'  => \Webklex\PHPIMAP\Events\MessageDeletedEvent::class,
					'restored' => \Webklex\PHPIMAP\Events\MessageRestoredEvent::class,
				],
				"folder"  => [
					'new'     => \Webklex\PHPIMAP\Events\FolderNewEvent::class,
					'moved'   => \Webklex\PHPIMAP\Events\FolderMovedEvent::class,
					'deleted' => \Webklex\PHPIMAP\Events\FolderDeletedEvent::class,
				],
				"flag"    => [
					'new'     => \Webklex\PHPIMAP\Events\FlagNewEvent::class,
					'deleted' => \Webklex\PHPIMAP\Events\FlagDeletedEvent::class,
				],
			],

			/*
			|--------------------------------------------------------------------------
			| Available masking options
			|--------------------------------------------------------------------------
			|
			| By using your own custom masks you can implement your own methods for
			| a better and faster access and less code to write.
			|
			| Checkout the two examples custom_attachment_mask and custom_message_mask
			| for a quick start.
			|
			| The provided masks below are used as the default masks.
			 */
			'masks'       => [
				'message'    => \Webklex\PHPIMAP\Support\Masks\MessageMask::class,
				'attachment' => \Webklex\PHPIMAP\Support\Masks\AttachmentMask::class,
			],
		];
	}
}
