<?php

namespace WpifyErp\Features;

use WpifyErp\Models\ClientModel;
use WpifyErp\Models\ProjectModel;
use WpifyErp\Repositories\ClientRepository;
use WpifyErp\Repositories\PermissionRepository;
use WpifyErp\Repositories\ProjectRepository;
use WpifyErp\Repositories\UserRepository;
use WpifyErpDeps\Wpify\CustomFields\CustomFields;

class Permissions {
	const ROLE_READER = 'reader';
	const ROLE_EDITOR = 'editor';
	const ROLE_MANAGER = 'manager';

	public function __construct(
		private CustomFields $custom_fields,
		private ClientRepository $client_repository,
		private ProjectRepository $project_repository,
		private UserRepository $user_repository,
		private PermissionRepository $permission_repository
	) {
		add_action( 'init', [ $this, 'register_custom_fields' ] );
		add_action( 'init', [ $this, 'custom_roles' ] );
		add_action( 'wpify_model_repository_save_insert', [ $this, 'add_permissions_item_created' ], 10, 4 );
		add_filter( 'wpify_model_refresh_model_after_save', '__return_false' );
	}

	public function custom_roles() {
		$roles   = [
			[
				'role'         => 'wpify_erp_internal',
				'name'         => 'Internal',
				'capabilities' => [
					'read'    => true,
					'level_0' => true,
				],
			],
		];
		$version = md5( json_encode( $roles ) );
		if ( get_option( 'wpify_erp_custom_roles_version' ) !== $version ) {
			foreach ( $roles as $role ) {
				add_role( $role['role'], $role['name'], $role['capabilities'] );
			}

			update_option( 'wpify_erp_custom_roles_version', $version );
		}
	}

	public function get_roles() {
		return [
			[
				'value' => self::ROLE_READER,
				'label' => 'Reader',
			],
			[
				'value' => self::ROLE_EDITOR,
				'label' => 'Editor',
			],
			[
				'value' => self::ROLE_MANAGER,
				'label' => 'Manager',
			],
		];
	}

	public function get_modules() {
		return [
			[
				'value' => 'task',
				'label' => 'Task',
			],
			[
				'value' => 'document',
				'label' => 'Document',
			],
			[
				'value' => 'time_entry',
				'label' => 'Time Entry',
			],
			[
				'value' => 'attachment',
				'label' => 'Attachment',
			],
			[
				'value' => 'activity_log',
				'label' => 'Activity Log',
			],
		];
	}

	public function register_custom_fields() {
		global $pagenow;

		if ( ! is_admin() || ( $pagenow !== 'users.php' && $pagenow !== 'user-new.php' && $pagenow !== 'profile.php' && $pagenow !== 'user-edit.php' ) ) {
			return;
		}
		$items    = [];
		$projects = $this->project_repository->find_all();
		$clients  = $this->client_repository->find_all();
		foreach ( $clients as $client ) {
			$items[] = [
				'value' => 'client_' . $client->id,
				'label' => sprintf( 'Client - %s', $client->title ),
			];
		}
		foreach ( $projects as $project ) {
			$items[] = [
				'value' => 'project_' . $project->id,
				'label' => sprintf( 'Project - %s', $project->name ),
			];
		}

		$this->custom_fields->create_user_options(
			array(
				'items' => array(
					[
						'id'    => 'wpify_erp_permissions',
						'title' => 'ERP Permissions',
						'type'  => 'multi_group',
						'items' => [
							[
								'title'   => 'Object',
								'type'    => 'select',
								'id'      => 'object',
								'options' => $items,
							],
							[
								'title'   => 'Role',
								'type'    => 'select',
								'id'      => 'role',
								'options' => $this->get_roles(),
							],
							[
								'type'  => 'group',
								'id'    => 'modules',
								'label' => 'Modules',
								'items' => array_map( fn( $module ) => [
									'id'      => $module['value'],
									'label'   => $module['label'],
									'type'    => 'select',
									'options' => $this->get_roles(),
								], $this->get_modules() ),
							],
						],
					],
				),
			) );
	}

	public function add_permissions_item_created( $item, $repo ) {
		$type = null;
		if ( is_a( $item, ClientModel::class ) ) {
			$type = 'client';
		}
		if ( is_a( $item, ProjectModel::class ) ) {
			$type = 'client';
		}
		if ( ! $type ) {
			return;
		}

		$permission              = $this->permission_repository->create();
		$permission->object_id   = $item->id;
		$permission->object_type = $type;
		$permission->role        = self::ROLE_MANAGER;
		$permission->user_id     = get_current_user_id();

		$this->permission_repository->save( $permission );
	}
}
